/* Debug Message Demo
 * http://frank.circleofcurrent.com/musicclock/
 * Copyright (c) 2011 Frank Zhao
 *
 * this example is based off http://www.pjrc.com/teensy/usb_serial.html
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

// please see http://www.nongnu.org/avr-libc/user-manual/modules.html for info on these modules

#include <avr/io.h> // allows access to AVR built-in peripherals and registers
#include <avr/pgmspace.h> // allows access to flash memory
#include <util/delay.h> // provides delays
#include <stdio.h> // allows formatted debug output
#include <avr/power.h> // used for setting clock prescaler
#include "usb_serial/usb_serial.h" // provides USB-to-serial functionality

// this is a wrapper for usb_putchar that allows me to use printf from stdio.h
static inline int debug_putchar(char c, FILE* stream)
{
    return usb_serial_putchar(c); // refer to http://www.pjrc.com/teensy/usb_serial.html
}

// setup stream for printf from stdio.h
static FILE mystdout = FDEV_SETUP_STREAM(debug_putchar, NULL, _FDEV_SETUP_WRITE);

int main()
{
    clock_prescale_set(clock_div_2); // this is because we are running at 3.3V
    // see http://www.nongnu.org/avr-libc/user-manual/group__avr__power.html and http://www.pjrc.com/teensy/3volt.html
    
    stdout = &mystdout; // setup stream for printf from stdio.h
    
    usb_init(); // initialize usb obviously
    
    while (usb_configured() == 0) _delay_ms(5); // waits until USB is ready
    
    _delay_ms(5000); // 5 seconds for you to connect hyperterminal
    
    // here's the start of the demonstration
    printf("hello world\r\n");
    
    // let's output a number
    int i = 5;
    printf("i = %d\r\n", i);
    
    // let's output a string
    char* str = "this is a string";
    printf("%s\r\n", str);
    
    while (1); // end the program safely by infinit loop
    return 0;
}